
//This file parses a sample schema document and validates an instance
  
#include <iostream>
#include <fstream>
#include <string>
#include <xmlpull/XmlPullParser.h>
#include <xmlpull/XmlPullParserException.h>
#include <schemaparser/SchemaParser.h>
#include <schemaparser/SchemaValidator.h>
#include <schemaparser/TypeContainer.h>
#include <schemaparser/SchemaParserException.h>
using namespace std;

void printComplexType (ComplexType * ct, TypeContainer * t,
			  SchemaParser * sp);
int 
main (int argc, char *argv[]) 
{
  ifstream schfs;
  ifstream insfs;
  if (argc <= 1)
    
    {
      cout << "Usage: schema <schema_file_name> [-i] <schema_instance_name>"
           <<endl;
      return 1;
    }
  
  else
    
    {
      schfs.open (argv[1]);
      if (schfs.fail ())
	
	{
	  cout << "An Error occrred while opening " << argv[1] << endl;
	  return 1;
	}
    }
  XmlPullParser * xpp = new XmlPullParser (schfs);
  xpp->setFeature (FEATURE_PROCESS_NAMESPACES, true);
  xpp->require (XmlPullParser::START_DOCUMENT, "", "");
  while (xpp->getEventType () != xpp->END_DOCUMENT)
    
    {
      xpp->nextTag ();
      if (xpp->getEventType () == xpp->START_TAG 
          &&xpp->getName () == "schema")
	break;
    }
  if (xpp->getName () != "schema")
    return 1;			//something wrong!
  try{


    SchemaParser * sp = new SchemaParser (xpp);
  
    if (sp->parseSchemaTag ())
      {
        cout << "List of parsed types in the namespace " <<sp->getTargetNamespace() << endl;
        cout << "Number of types "<<sp->getNumTypes()<<endl;
#ifdef LOGGING
        //      sp->print (cout);
#endif
      }
    else return 1;
  
    if (argc == 4)
    
      {
        insfs.open (argv[3]);	//open the schema instance file
        if (insfs.fail ())
	
          {
            cout << "An Error occrred while opening " << argv[3] << endl;
            return 1;
          }
        delete xpp;
        xpp = new XmlPullParser (insfs);
        xpp->setFeature (FEATURE_PROCESS_NAMESPACES, true);
        xpp->require (XmlPullParser::START_DOCUMENT, "", "");
        SchemaValidator * sv= new SchemaValidator(sp);
        while (xpp->getEventType () != xpp->END_DOCUMENT)
	
          {
            xpp->nextTag ();
            if (xpp->getEventType () == xpp->END_DOCUMENT)
              break;
            Qname elemName (xpp->getName ());
            int elemId = sp->getElementId (elemName);
            Element * e = sp->getElement (elemId);
            int typeId = e->getType () ;
            TypeContainer * t = sv->validate (xpp, typeId);
            cout << elemName.getNamespace () << ":" << elemName. 
              getLocalName ();
            const XSDType * pType = sp->getType (typeId);
            if (pType->isSimple ())
	    
              {
                SimpleType * st = (SimpleType *) pType;
                cout << st->getName () << ":" << t->getValue ();
              }
	  
            else
	    
              {
                ComplexType * ct = (ComplexType *) pType;
                printComplexType (ct, t, sp);
              }
          }
      }
    return 0;
  }
  catch (SchemaParserException spe)
    {
      cout<<"An Exception occurred ...@"<<spe.line
          <<":"<<spe.col<<endl;

      cout<<spe.description<<endl;
    }
  catch (XmlPullParserException xpe)
    {
      cout<<"An Exception occurred ...@"<<xpe.line
          <<":"<<xpe.col<<endl;

      cout<<xpe.description<<endl;
    }
    
}


void 
printComplexType (ComplexType * ct, TypeContainer * t, SchemaParser * sp) 
{
  int i;
  cout << endl;
  for (i = 0; i < ct->getNumAttributes (); i++)
    
    {
      TypeContainer * tmp =
	t->getAttributeContainer (ct->getAttributeName (i));
      
      //if(tmp->getTypeId()==XSDType::STRING)
        cout << "@" << ct->getAttributeName (i) << ":" 
             << *((string *) tmp-> getValue ()) << endl;
        // else
        //cout << "@" << ct->getAttributeName (i) << ":" << endl; //ok,,this is just a sample ,in a real 
      //example you wouild have to typecast is to the type of the attribute and print it.

    }

  if(ct->getContentModel()==ct->SIMPLE)
    cout<< *((string *) t-> getValue ()) << endl;

  for (i = 0; i < ct->getNumChildren (); i++)
    
    {
      TypeContainer * tmp = 0;
      
      do
	
	{
	  tmp = t->getChildContainer (ct->getChildName (i));
	  
          //if more than one occurences of the element are found then
          //the same call will return the successive instances of the element
          if (tmp == 0)
	    continue;
	  cout << ct->getChildName (i) << ":";
	  const XSDType * pType = 
	    sp->getType (ct->getChildType (i));
	  if (pType && !pType->isSimple ())
	    
	    {
	      printComplexType ((ComplexType *) pType, tmp, sp);
	    }
	  
	  else
	    
	    {
	      int typeId = 
		(pType) ? ((SimpleType *) pType)->getBaseType () : ct->
		getChildType (i);
	      if (typeId == XSDType::DATE)
		
		{
		  cout << *((string *) (tmp->getValue ())) << endl;
		}
	      
	      else if (typeId == XSDType::INTEGER || typeId == XSDType::INT)
		
		{
		  cout << *((int *) (tmp->getValue ())) << endl;
		}
	      
	      else if (typeId == XSDType::DECIMAL
		       || typeId == XSDType::FLOAT  ||typeId ==
		       XSDType::DOUBLE)
		
		{
		  cout << *((double *) (tmp->getValue ())) << endl;
		}
	      
	      else if (typeId == XSDType::POSINT || typeId == XSDType::LONG 
		       ||typeId == XSDType::ULONG)
		
		{
		  cout << *((long *) (tmp->getValue ())) << endl;
		}
              else 
                {
		  cout << *((string *) (tmp->getValue ())) << endl;
	   
                }
              
	    }
	}
      while (tmp != 0);
    }
}


