/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */

#include <gsoap.h>
#include <sstream>
#include <map>
int *isDefnGenerated;
/*
  This function generates the header file to be processed by the gsoap
  pre-processor
  */
bool
gsoap_generateHeaderFile (WsdlParser & wParser, Soap & soap, ostream * ofs)
{

/*
  1. Generate typedefs for all primitive types
     ex typedef char*  xsd__string;
  2. Generate structs for all complex types
  3. Generate the gsoap directive for each operation
  4. Generate prototype for each operation.
*/
	//Binding & bn = wParser.getBinding (0);
	list < const Binding *>*plBinding = wParser.getBindings ();
	const Binding *bn = plBinding->front ();
	int soap_binding_elem = 0, i = 0, nSchemas = 0;
	bool status = true, headerDefined = false;
	char *location = 0;
	const SchemaParser **pSchArray = 0;
	string sPrefix;
	soap_binding_elem = soap.getElementName (bn->getBindingInfo ());

	if (soap_binding_elem == 0)
		return false;	//only soap binding supported

	soap.getServiceLocation (bn->getServiceExtId (), &location);
	if (location == 0)
		return false;	//a binding must have a location

	//generate the typedefs for primitive types
#ifdef _WIN32
	gsoap_CopyFile2Stream (string ("examples/gsoap/typedefs.h").c_str (),
			       ofs);
#else
	gsoap_CopyFile2Stream ((string (DATADIR) + "typedefs.h").c_str (),
			       ofs);
#endif


	pSchArray = wParser.getSchemaParsers ();
	nSchemas = wParser.getNumSchemas ();

//create a prefix for wsdl targetNamespace
	//gsoap_getNamespacePrefix (wParser.getTargetNamespace ());
	while (nSchemas--)

	{
		// generate the structs for types defined in each schema

		sPrefix =
			gsoap_getNamespacePrefix (pSchArray[nSchemas]->
						  getTargetNamespace ());
		if (pSchArray[nSchemas]->getNumTypes () > 0)
			gsoap_createDirective (sPrefix, "schema", "namespace",
					       pSchArray[nSchemas]->
					       getTargetNamespace (), ofs);

		isDefnGenerated =
			new int[pSchArray[nSchemas]->getNumTypes ()];
		for (i = 0; i < pSchArray[nSchemas]->getNumTypes (); i++)
			isDefnGenerated[i] = false;
		//need to come up with a better way of keeping track of which structs are generated

		list < const XSDType *>*pLTypes =
			pSchArray[nSchemas]->getAllTypes ();
		list < const XSDType *>::iterator tli;

		for (tli = pLTypes->begin (), i = 0;
		     tli != pLTypes->end () && status; tli++, i++)
		{
			if (isDefnGenerated[i])
				continue;
			const XSDType *pType = *tli;

			if (pType->isAnonymous () ||
			    (pType->getQname ()).getNamespace () !=
			    pSchArray[nSchemas]->getTargetNamespace ())
				continue;

			status = gsoap_createStruct (pType,
						     pSchArray[nSchemas],
						     ofs);
			isDefnGenerated[i] = true;
		}

		// generate the structs for elements defined in each schema
		for (i = 0; i < pSchArray[nSchemas]->getNumElements (); i++)
		{
			Element *e = pSchArray[nSchemas]->getElement (i);
			if (!pSchArray[nSchemas]->isBasicType (e->getType ()))
			{
				XSDType *elType =
					(XSDType *) pSchArray[nSchemas]->
					getType (e->getType ());
				if (elType->getName () == e->getName ())	//element and type have same name.will result in names clash
					continue;
				elType->setName (e->getName ());
				gsoap_createStruct ((const XSDType *) elType,
						    pSchArray[nSchemas], ofs);
			}


		}
		delete isDefnGenerated;
		delete pLTypes;
		if (!status)
			return false;
	}


	int numOps = bn->numOps ();
	i = 0;
	while (i < numOps)

	{
		int soapBindingId = 0, inId = 0, inHeader = 0, outId =
			0, outHeader = 0, faultId = 0;
		const Operation *op = 0;
		char *soapAction = 0, *Ns = 0;
		int style, use;
		int inParams = 0, outParams = 0;
		int j = 0, typeId = 0, elemId = -1;
		XSDType *pType = 0;

//get the operation's wsdl id
//and its soap binding element ids
		op = bn->getOperation (i);
		const int *bindings = 0;
		int nBindings = 0;
		nBindings = bn->getOpBinding (i, &bindings);
		soapBindingId = bindings[0];

		nBindings = bn->getInputBinding (i, &bindings);

		if (nBindings > 1)
		{
			if (soap.isSoapBody (bindings[0]))
				inId = bindings[0];
			else if (soap.isSoapHeader (bindings[0]))
				inHeader = bindings[0];


			if (soap.isSoapBody (bindings[1]))
				inId = bindings[1];
			else if (soap.isSoapHeader (bindings[1]))
				inHeader = bindings[1];
		}
		else
			inId = bindings[0];

		nBindings = bn->getOutputBinding (i, &bindings);
		if (nBindings > 1)
		{
			if (soap.isSoapBody (bindings[0]))
				outId = bindings[0];
			else if (soap.isSoapHeader (bindings[0]))
				outHeader = bindings[0];

			if (soap.isSoapBody (bindings[1]))
				outId = bindings[1];
			else if (soap.isSoapHeader (bindings[1]))
				outHeader = bindings[1];
		}
		else
			outId = bindings[0];

		nBindings = bn->getFaultBinding (i, &bindings);
		faultId = bindings[0];

/*
 * generate the gsoap SOAP_ENV_Header struct
 * do it only once ,because there can be only one definition
 */
		if (!headerDefined && (outHeader || inHeader))
		{
			int schemaId, headerTypeId;
			soap.getSoapHeaderInfo (inHeader, schemaId,
						headerTypeId);
			gsoap_generate_HeaderStruct (headerTypeId,
						     (wParser.
						      getSchemaParser
						      (schemaId)), ofs);
			headerDefined = true;
		}

		//create the gsoap's namespace directive for each operation
		soap.getSoapOperationInfo (soapBindingId, &soapAction, style);
		soap.getSoapBodyInfo (inId, &Ns, use);
		if (!Ns)
		{
			sPrefix =
				gsoap_getNamespacePrefix (wParser.
							  getTargetNamespace
							  ());
			gsoap_createDirective (sPrefix, "service",
					       "namespace",
					       wParser.getTargetNamespace (),
					       ofs);
		}
		else
		{
			sPrefix = gsoap_getNamespacePrefix (Ns);
			gsoap_createDirective (sPrefix, "service",
					       "namespace", Ns, ofs);
		}

		const Message *inMessage = op->getMessage (Input);
		const Message *outMessage = op->getMessage (Output);

		if (use == Soap::LITERAL)
			gsoap_createDirective (sPrefix, "service",
					       "method-encoding", "literal",
					       ofs);
		else
			gsoap_createDirective (sPrefix, "service",
					       "method-encoding", "encoded",
					       ofs);

		if (style == Soap::DOC)
			gsoap_createDirective (sPrefix, "service",
					       "method-style", "document",
					       ofs);
		else
			gsoap_createDirective (sPrefix, "service",
					       "method-style", "rpc", ofs);


		string opName = op->getName ();
		if (soapAction)
			gsoap_createDirective (sPrefix, "service",
					       "method-action",
					       opName + " " + soapAction,
					       ofs);
                if (use == Soap::LITERAL)
                {
	           gsoap_createRemoteMethod ("__"+sPrefix, opName, ofs);
                }
                else 
                 {
              	   gsoap_createRemoteMethod (sPrefix, opName, ofs);
                 }
		inParams = inMessage->getNumParts ();

//add all input params to remote method
		for (j = 0; j < inParams; j++)
		{
			typeId = inMessage->getPartType (j);
			const SchemaParser *sParser =
				wParser.getSchemaParser (inMessage->
							 getPartContentSchemaId
							 (j));
			string name;
			if (inMessage->getPartRefType (j) == Elem)
			{
				Element *e = sParser->getElement (typeId);
				name = "_" + sPrefix + "__" + e->getName ();
				typeId = e->getType ();
			}
			else
			{
				name = inMessage->getPartName (j);
			}
			gsoap_addInParam (gsoap_generateQualifiedType
					  (typeId, sParser), name, ofs);
		}

//add all output params to remote method
		outParams = outMessage->getNumParts ();

		if (style==Soap::RPC)
		{
			//create a struct inline
			ostringstream structDefn;
			structDefn << "struct " << sPrefix << "__" << opName
				<< "Response" << " {" << dbsp;
			for (j = 0; j < outParams; j++)
			{
			
			 typeId = outMessage->getPartType (j);
			 const SchemaParser *sParser =
			 wParser.getSchemaParser (outMessage->
				                      getPartContentSchemaId (j));
			string name;
			if (outMessage->getPartRefType (j) == Elem)
			{
				Element *e = sParser->getElement (typeId);
				name = "_" + sPrefix + "__" + e->getName ();
				typeId = e->getType ();
			}
			else
			{
				name = "_"+outMessage->getPartName (j);
				typeId=outMessage->getPartType(j);
			}
			structDefn<<gsoap_generateQualifiedType(typeId,sParser)<<"*"
			          <<dbsp<<name<<";"<<dbsp;
			}
			structDefn << "}";
			gsoap_addOutParam (structDefn.str (), "out", ofs);
		}
		else  
		{
			//style==DOC.
         	      if(outParams>1) 
		          return false; //tmp hack
            


			for (j = 0; j < outParams; j++)
			{
			
			 typeId = outMessage->getPartType (j);
			 const SchemaParser *sParser =
			 wParser.getSchemaParser (outMessage->
				                      getPartContentSchemaId (j));
			string name;
			if (outMessage->getPartRefType (j) == Elem)
			{
				Element *e = sParser->getElement (typeId);
				name = "_" + sPrefix + "__" + e->getName ();
				typeId = e->getType ();
			}
			else
			{
				name = "_"+outMessage->getPartName (j);
				typeId=outMessage->getPartType(j);
			}
			gsoap_addOutParam (gsoap_generateQualifiedType(typeId,sParser), name, ofs);
		}
	   }
		


		gsoap_finishRemoteMethod (ofs);
		i++;

	}
	return true;
}
