/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <schemaparser/SchemaParser.h>

#ifdef LOGGING
void SchemaParser::print(ostream & out) 
{
    m_typesTable.printTypes(out);
}


void TypesTable::printTypes(ostream & out)
{
    out << numTypes << endl;
    for (int i = 0; i < numTypes; i++)
    {
        typesArray[i]->print(out);
        out << blk;
    }
}
void
  ComplexType::print (ostream & out)
{
  out << name << dbsp << typeId << dbsp << isSimple () << endl;
  out << contentModel << dbsp;
  if (contentModel == SIMPLE)
    out << contentTypeId;
  out << endl;

  out << num_atts << endl;
  list < Attribute >::iterator pAttr = att_list.begin ();
  while (pAttr != att_list.end ())
    {
      out << *pAttr;
      pAttr++;
    }
  out << num_elems << endl;
  list < Element >::iterator pElems = elem_list.begin ();
  while (pElems != elem_list.end ())
    {
      out << *pElems;
      pElems++;
    }

}


ostream & operator << (ostream & stream, Attribute & a)
{
  stream << "@" << a.getName () << dbsp << a.getType () << dbsp << dbsp;
  stream << a.isRequired () << dbsp;

  if (!(a.defaultVal ()).empty ())
    {
      stream << 1 << dbsp << a.defaultVal ();
  } else
    stream << 0 << dbsp;
  if (!(a.fixedVal ()).empty ())
    {
      stream << 1 << dbsp << a.fixedVal ();
  } else
    stream << 0 << dbsp;
  stream << endl;

  return stream;

}


ostream & operator << (ostream & stream, Element & e)
{
  stream << e.getName () << dbsp << e.getType () << dbsp << dbsp;
  stream << e.min () << dbsp << e.max () << dbsp;
  if (!(e.defaultVal ()).empty ())
    {
      stream << 1 << dbsp << e.defaultVal ();
  } else
    stream << 0 << dbsp;
  if (!(e.fixedVal ()).empty ())
    {
      stream << 1 << dbsp << e.fixedVal ();
  } else
    stream << 0 << dbsp;

  stream << endl;

  return stream;
}

void
  SimpleType::print (ostream & out)
{

  out << name << dbsp << typeId << dbsp << isSimple () << endl;
  out << basetype << endl;
  out << restriction << dbsp;
  out << numFacets << endl;
  for (int j = 0; j < numFacets; j++)
    {
      out << facetId[j] << dbsp;
      if (facetId[j] == ENUM)
	{
	  out << facetValue.numEnums << dbsp;
	  for (list < string >::iterator pEnums = enum_list.begin ();
	       pEnums != enum_list.end (); pEnums++)
	    out << *pEnums << dbsp;
	}
      out << endl;
    }
  out << endl;
//TODO need to o/p other facets
}

#endif
