/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */

#include <schemaparser/SimpleType.h>

//TODO .Need to add checks for validating the occurence of consecutive facets

SimpleType::SimpleType(string ns)
  :nsUri(ns)
{
  basetype = 0;
  contentModel = SIMPLE;
  restriction = false;
  anonymous = false;
  for (int j = 0; j < 5; j++)
    facetId[j] = NONE;
  numFacets = 0;
  validFacets = new int[ANYURI + 1];

  //This table maintains a flag which indicates what facets are valid  for a simple type
  validFacets[STRING] = validFacets[NMTOKEN] = validFacets[NMTOKENS] =
    validFacets[TOKEN] =
    LENGTH | MINLEN | MAXLEN | ENUM | WSP | PATTERN;
  validFacets[INTEGER] = validFacets[INT] = validFacets[UINT] =
    validFacets[BYTE] = validFacets[POSINT] =
    ENUM | WSP | MAXEX | MINEX | MAXINC | MININC | TOT | FRAC |
    PATTERN;
  validFacets[LONG] = validFacets[ULONG] = validFacets[DECIMAL] =
    validFacets[INT];
  validFacets[SHORT] = validFacets[USHORT] = validFacets[INT];
  validFacets[FLOAT] = validFacets[DOUBLE] =
    ENUM | WSP | MAXEX | MINEX | MAXINC | MININC | PATTERN;
  validFacets[BOOLEAN] = WSP | PATTERN;
  validFacets[TIME] = validFacets[DATETIME] = validFacets[DATE] =
    ENUM | WSP | MAXEX | MINEX | MAXINC | MININC | PATTERN;
  validFacets[QNAME] = validFacets[NCNAME] = validFacets[ANYURI] =
    LENGTH | MINLEN | MAXLEN | ENUM | WSP | PATTERN;
  validFacets[ANY] = validFacets[ANYTYPE] = NONE | PATTERN;
  validFacets[BASE64BIN] = validFacets[STRING];

  //initialize the union??TODO
  facetValue.numEnums = 0;
}

SimpleType::~SimpleType() {
  delete[] validFacets;
}



string 
SimpleType::getName() const
{
  return name;
}

Qname 
SimpleType::getQname() const
{
        Qname qn(name);
        qn.setNamespace(nsUri);
        return qn;
}

int 
SimpleType::getBaseType()const 
{
        return basetype;
}


int 
SimpleType::getContentModel() const
{
        return contentModel;
}

int 
SimpleType::getTypeId() const
{
        return typeId;
}

void 
SimpleType::setTypeId(int id)
{
        typeId = id;
}


bool 
SimpleType::isAnonymous() const
{
        return anonymous;
}

void 
SimpleType::setAnonymous(bool flag)
{
        anonymous = flag;
}

void 
SimpleType::setName(string s)
{
          name = s;
}

void 
SimpleType::setContentModel(int cm)
{
          contentModel = cm;
}

void 
SimpleType::setBaseType(int id)
{
        basetype = id;
        restriction = true;
}


//This function also sets the facet if valid
bool SimpleType::isvalidFacet(string facet)
{
    if (basetype == 0)
        error("isValidFacet:Unknown base type");
    if (numFacets >= 5)
        error("Cannot support more than 5 facets for a restriction");
    if (facet == "length")
        facetId[numFacets] = LENGTH;

    else if (facet == "minLength")
        facetId[numFacets] = MINLEN;

    else if (facet == "maxLength")
        facetId[numFacets] = MAXLEN;

    else if (facet == "enumeration")
        facetId[numFacets] = ENUM;

    else if (facet == "whiteSpace")
        facetId[numFacets] = WSP;

    else if (facet == "pattern")
        facetId[numFacets] = PATTERN;

    else if (facet == "maxInclusive")
        facetId[numFacets] = MAXINC;

    else if (facet == "maxExclusive")
        facetId[numFacets] = MAXEX;

    else if (facet == "minInclusive")
        facetId[numFacets] = MININC;

    else if (facet == "minExclusive")
        facetId[numFacets] = MINEX;

    else if (facet == "totalDigits")
        facetId[numFacets] = TOT;

    else if (facet == "fractionDigits")
        facetId[numFacets] = FRAC;

    else
    {
        facetId[numFacets] = NONE;
        return false;
    }
    if (validFacets[basetype] | facetId[numFacets])
        return true;
}


//TODO ..need to add checks to see which facets can appear together.
void SimpleType::setFacetValue(string val)
{
    int num = -1;
    switch (facetId[numFacets])
    {
        case ENUM:
            facetValue.numEnums++;
            enum_list.push_back(val);
            break;
        case PATTERN:
            facetValue.pattern = (const char*) val.c_str();
            break;
        case WSP:
            if (val == "preserve")
                facetValue.wsp = PRESERVE;

            else if (val == "collapse")
                facetValue.wsp = COLLAPSE;

            else if (val == "replace")
                facetValue.wsp = REPLACE;

            else
                error("Invalid facet value for whitespace");
            break;

//all other facet values are numbers
        default:
            num = parseInt(val);
            break;
    }
    switch (facetId[numFacets])
    {
        case MAXEX:
            facetValue.valRange.maxex = num;
            break;
        case MAXINC:
            facetValue.valRange.maxinc = num;
            break;
        case MININC:
            facetValue.valRange.mininc = num;
            break;
        case MINEX:
            facetValue.valRange.minex = num;
            break;
        case LENGTH:
            facetValue.length = num;
            break;
        case MINLEN:
            facetValue.lenRange.minlen = num;
            break;
        case MAXLEN:
            facetValue.lenRange.maxlen = num;
            break;
        case TOT:
            facetValue.tot = num;
            break;
        case FRAC:
            facetValue.frac = num;
            break;
        default:
            break;
    }

//In case of  enum the new enum value does not define a new facet
    if (!((numFacets != 0) && (facetId[numFacets - 1] == ENUM)))
        numFacets++;
}


bool SimpleType::isValidInt(int val)
{
    if (basetype != XSDType::INT)
        return false;
    int numdigits = 1, tmp = val;
    bool valid = false;
    while (tmp = tmp / 10)
        numdigits++;
    for (int i = 0; i < numFacets; i++)
    {
        switch (facetId[i])
        {
            case MAXEX:
                if (val < facetValue.valRange.maxex)
                    valid = true;

                else
                    valid = false;
                break;
            case MAXINC:
                if (val <= facetValue.valRange.maxinc)
                    valid = true;

                else
                    valid = false;
                break;
            case MININC:
                if (val >= facetValue.valRange.mininc)
                    valid = true;

                else
                    valid = false;
                break;
            case MINEX:
                if (val > facetValue.valRange.minex)
                    valid = true;

                else
                    valid = false;
                break;
            case LENGTH:
                if (numdigits == facetValue.length)
                    valid = true;

                else
                    valid = false;
                break;
            case MINLEN:
                if (numdigits >= facetValue.lenRange.minlen)
                    valid = true;

                else
                    valid = false;
                break;
            case MAXLEN:
                if (numdigits <= facetValue.lenRange.maxlen)
                    valid = true;

                else
                    valid = false;
                break;
            default:
                valid = false;
        }
    }
    return valid;
}


bool SimpleType::isValidFloat(float val)
{

//TODO
    return false;
}


bool SimpleType::isValidString(string val)
{
    if (basetype != XSDType::STRING)
        return false;
    int strlen = val.length();
    bool valid = false;
    list < string >::iterator pEnums;
    for (int i = 0; i < numFacets; i++)
    {
        switch (facetId[i])
        {
            case LENGTH:
                if (strlen == facetValue.length)
                    valid = true;

                else
                    valid = false;
                break;
            case MINLEN:
                if (strlen >= facetValue.lenRange.minlen)
                    valid = true;

                else
                    valid = false;
                break;
            case MAXLEN:
                if (strlen <= facetValue.lenRange.maxlen)
                    valid = true;

                else
                    valid = false;
                break;
            case ENUM:
                for (pEnums = enum_list.begin(); pEnums != enum_list.end();
                    pEnums++)
                if (*pEnums == val)
                    valid = true;
                break;
            case PATTERN:
                valid = true;                     //TODO
                break;
            default:
                valid = false;
        }
    }
    return valid;
}


//This function returns the facet values of the requested facet type
// if the type is restricted by it.
// For enum it returns(via val*) a ptr to a list of strings
// For all other facets related to length,its returns(via val*) an int ptr because
// For inclusive and exclusive facets it returns (via val*) ptr to the value

//TODO add cases to return pointers to other facet values
bool SimpleType::getFacetValue(int facet, void **val)
{
    *val = 0;
    bool isFacetPresent = false;
    for (int i = 0; i < numFacets && !isFacetPresent; i++)
        isFacetPresent = (facetId[i] == facet);
    if (!isFacetPresent)
        return false;
    switch (facet)
    {
        case ENUM:
            *val = (void *) &enum_list;
            return true;
            break;
        default:
            val = 0;
            return false;
    }
}


void SimpleType::error(string msg)
{
    msg += "SimpleType::error()";
    SchemaParserException spe(msg);
    throw spe;
    return;
}
