/* 
 * wsdlpull- A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */

#include <schemaparser/TypeContainer.h>

TypeContainer::TypeContainer(int id, const SchemaParser * sp)
                           :sParser(sp), typeId(id),nTc(0)
{
  /**
   * This is a union so setting one pointer
   *  will set all to 0 
   */
    
  Value.sValue=0;
  
}


TypeContainer::~TypeContainer()
{

  if(nTc>0) {

    /* 
       delete all sub type containers
    */
    for (int i = 0; i < nTc; i++)
      delete tcTable[i];
  
  }else {

    /* 
       delete the values if it is an atomic type container
    */
    deleteValue();

  }
}


TypeContainer*
TypeContainer::getAttributeContainer(string elemName,
                                     bool create)
{
  TypeContainer *container = 0;
  if ((container = attributeList[elemName]) != 0)
    return container;
  if (!create)
    return container;

  /*
    Create and return a Type Container
  */
  const XSDType *pType = sParser->getType(typeId);
  if (pType != 0 && !pType->isSimple())
    {
      ComplexType *cType = (ComplexType *) pType;
      for (int i = 0; i < cType->getNumAttributes(); i++)
        {
          if (cType->getAttributeName(i) == elemName)
            {
              container =
                new TypeContainer(cType->getAttributeType(i), sParser);
              tcTable[nTc++] = container;
              break;
            }
        }
    }
  attributeList[elemName] = container;
  return container;
}


TypeContainer *TypeContainer::getChildContainer(string elemName,
                                                bool create)
{
  Containers *cs = 0;
  TypeContainer *tC = 0;
  if (!create)
    {

      /*
        Each time this method is called the next occurence  of
        the child container is returned .Once all the occurrences
        of the child container have been accessed ,rewind to the 1st
        occurrence
      */
      if ((cs = childList[elemName]) != 0)
        {
          if (cs->count >= cs->num)
            {
              cs->count = 0;
              return 0;
            }

          else
            return cs->tc[cs->count++];
        }

      else
        return 0;
    }

  else
    {

      /*
        Create and return a Type Container
      */
      cs = childList[elemName];
      if (!cs)
        {
          cs = new Containers;
          cs->count = cs->num = 0;
          childList[elemName] = cs;
        }
      const XSDType *pType = sParser->getType(typeId);
      if (pType != 0 && !pType->isSimple())
        {
          ComplexType *cType = (ComplexType *) pType;
          for (int i = 0; i < cType->getNumChildren(); i++)
            {
              if (cType->getChildName(i) == elemName)
                {
                  tC = new TypeContainer(cType->getChildType(i),
                                         sParser);
                  tcTable[nTc++] = tC;
                  break;
                }
            }
        }
      cs->tc[cs->num] = tC;
      cs->num++;
      return tC;
    }
}


void *TypeContainer::getValue()
{
  if(!Value.sValue)
    return 0; //this is an error
  
  const XSDType *pType = sParser->getType(typeId);
  int id = 0;
  if (pType == 0)
    {

      /*
        Its a primitive XSD type
      */
      id = typeId;
    }

  else
    {

      /* 
         It could be a complex type with
         simple content or a schema defined simpleType
      */
      if (pType->isSimple() == false)
        id = ((ComplexType *) pType)->getContentType();

      else
        id = ((SimpleType *) pType)->getBaseType();
    }
  switch (id)
    {

    case XSDType::INT:
    case XSDType::INTEGER:
      return (void *) Value.iValue;
    case XSDType::LONG:
      return (void *) Value.lValue;
    case XSDType::POSINT:
    case XSDType::ULONG:
      return (void *) Value.ulValue;
    case XSDType::FLOAT:
      return (void *) Value.fValue;
    case XSDType::DOUBLE:
    case XSDType::DECIMAL:
      return (void *) Value.dbValue;
    case XSDType::BOOLEAN:
      return (void *) Value.bValue;
    case XSDType::QNAME:
      return (void *) Value.qnValue;
    case XSDType::STRING:
    case XSDType::ANYTYPE:
    case XSDType::ANYURI:
    case XSDType::NMTOKEN:
    case XSDType::NMTOKENS:
    case XSDType::NCNAME:
    case XSDType::DATE:
    case XSDType::DATETIME:
    case XSDType::TIME:
    default:
      return (void *) Value.sValue;

    }
}


void
TypeContainer::deleteValue()
{
  if(!Value.sValue)
    return;
  
  const XSDType *pType = sParser->getType(typeId);
  int id = 0;
  if (pType == 0)
    {

      /*
        Its a primitive XSD type
      */
      id = typeId;
    }

  else
    {

      /* 
         It could be a complex type with
         simple content or a schema defined simpleType
      */
      if (pType->isSimple() == false)
        id = ((ComplexType *) pType)->getContentType();

      else
        id = ((SimpleType *) pType)->getBaseType();
    }
  switch (id)
    {

    case XSDType::INT:
    case XSDType::INTEGER:
      delete  Value.iValue;
    case XSDType::LONG:
       delete Value.lValue;
    case XSDType::POSINT:
    case XSDType::ULONG:
       delete Value.ulValue;
    case XSDType::FLOAT:
       delete Value.fValue;
    case XSDType::DOUBLE:
    case XSDType::DECIMAL:
       delete Value.dbValue;
    case XSDType::BOOLEAN:
       delete Value.bValue;
    case XSDType::QNAME:
       delete Value.qnValue;
    case XSDType::STRING:
    case XSDType::ANYTYPE:
    case XSDType::ANYURI:
    case XSDType::NMTOKEN:
    case XSDType::NMTOKENS:
    case XSDType::NCNAME:
    case XSDType::DATE:
    case XSDType::DATETIME:
    case XSDType::TIME:
    default:
       delete Value.sValue;
      
    }
}
