/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */

#include <sstream>
#include <wsdlparser/SoapExt.h>

/*
TODO
1.When the use is "encoded" the part must reference the an
  abstract type using the "type" attribute .
2.Only Soap encoding style is supported
3.Throw warnings when max limits are reached
*/

#include <iomanip>
Soap::Soap()
 :sNamespace("http://schemas.xmlsoap.org/wsdl/soap/")
{
  startId = idCounter = nOps = nMsgs = nLocation = nHeader = 0;
  wParser = 0;
  mySchemaParser = 0;
  mySchemaValidator = 0;
  for (int i = 0; i < MAX_OPS * MAX_PTYPES; i++)
    {
      m_body[i].urn = 0;
      if (i < MAX_OPS)
        m_ops[i].soapAction = 0;
      if (i < MAX_PTYPES)
        m_location[i] = 0;
    }
}


Soap::~Soap()
{
  if (mySchemaParser)
    delete mySchemaParser;
  if (mySchemaValidator)
    delete mySchemaValidator;
  int i = 0;
  for (i = 0; i < MAX_OPS; i++)
    if (m_ops[i].soapAction != 0)
      delete m_ops[i].soapAction;
  for (i = 0; i < MAX_PTYPES; i++)
    if (m_location[i] != 0)
      delete[] m_location[i];
  for (i = 0; i < MAX_OPS * MAX_PTYPES; i++)
    if (m_body[i].urn != 0)
      delete[] m_body[i].urn;
}


XmlPullParser * Soap::getExtensibilitySchema(void)
{
  ifstream ifs;
#ifdef _WIN32
  string path="src/schemas/";
#else
  string path=SCHEMADIR;
#endif
  path+="soap.xsd";
  ifs.open(path.c_str());
  if (ifs.fail())
    {
      cout << "An Error occrred while opening soap.xsd" << endl;
      cout <<"soap.xsd was not found in "<<path <<endl;
      return 0;
    }
  XmlPullParser * xParser = new XmlPullParser(ifs);
  xParser->setFeature(FEATURE_PROCESS_NAMESPACES, true);
  xParser->require(XmlPullParser::START_DOCUMENT, "", "");
  while (xParser->getEventType() != xParser->END_DOCUMENT)
    {
      xParser->nextTag();
      if (xParser->getEventType() == xParser->START_TAG
          &&xParser->getName() == "schema")
        return xParser;
    }
  return 0;
}


int  Soap::handleElement(int parent, XmlPullParser * xParser)
{
    if (idCounter >= MAX_EXT_XML)
        error("Reached max limit for soap elements");
    if (mySchemaParser == 0)
    {
        error("Could not parse soap extensibility elements");
        return 0;
    }
    string elemName = xParser->getName();
    int elemId = 0;
    Qname q(elemName);
    int typeId = mySchemaParser->getElementType(q);
    if (typeId == 0)
    {
        error("Unkown element");
        return 0;
    }
    TypeContainer * t = new TypeContainer(typeId, mySchemaParser);
    try{
       mySchemaValidator->validate(xParser, typeId, t);
      } catch (SchemaParserException spe)
        {
	  cout<<spe.description<<endl;
	  error("Encountered error while validating "+sNamespace +":"+elemName);
	}
    if (elemName == "binding")
        elemId = processBinding(t);

    else if (elemName == "operation")
        elemId = processOp(parent, t);

    else if (elemName == "body")
        elemId = processBody(parent, t);

    else if (elemName == "header")
        elemId = processHeader(parent, t);

    else if (elemName == "fault")
        elemId = processFault(parent, t);

    else if (elemName == "address")
        elemId = processAddress(parent, t);
    delete t;
    return elemId;
}


int
Soap::handleAttribute(int parent, string att,
XmlPullParser * xParser)
{
  return 0;
}


int  Soap::processBinding(TypeContainer * t)
{
  TypeContainer * temp = 0;
  if ((temp = t->getAttributeContainer("transport")) != 0)
    {
      string tp = *((string *) (temp->getValue()));
      if (tp == httpTransport)
        m_transport = HTTP;

      else
        m_transport = NONE;
    }

  else
    m_transport = HTTP;

  /*
   * Assume default transport as HTTP
   */
  if ((temp = t->getAttributeContainer("style")) != 0)
    {
      string style = *((string *) (temp->getValue()));
      if (style == "rpc")
        m_style = RPC;

      else
        m_style = DOC;
    }

  else
    m_style = DOC;
  Qname binding("binding");
  idTable[idCounter].typeId = mySchemaParser->getElementType(binding);
  idTable[idCounter].index = 0;
  idCounter++;
  return startId + idCounter - 1;
}


int  Soap::processOp(int parent, TypeContainer * t)
{
  TypeContainer * temp = 0;
  char *buf = 0;
  if (nOps >= MAX_OPS)
    return 0;
  if ((temp = t->getAttributeContainer("soapAction")) != 0)
    {
      string * s = (string *) (temp->getValue());
      buf = new char[s->length() + 1];
      strncpy(buf, s->c_str(), s->length() + 1);
      m_ops[nOps].soapAction = buf;
    }

  else
    m_ops[nOps].soapAction = 0;
  if ((temp = t->getAttributeContainer("style")) != 0)
    {
      string style = *((string *) (temp->getValue()));
      if (style == "rpc")
        m_ops[nOps].style = RPC;

      else
        m_ops[nOps].style = DOC;
    }

  else                                          //Use the binding element's style attribute
    m_ops[nOps].style = m_style;
  m_ops[nOps].wsdlOpId = parent;
  Qname oprn("operation");
  idTable[idCounter].typeId = mySchemaParser->getElementType(oprn);
  idTable[idCounter].index = nOps;
  idCounter++;
  nOps++;
  return startId + idCounter - 1;
}


int  Soap::processBody(int parent, TypeContainer * t)
{
  TypeContainer * temp = 0;
  string use;
  char *buf = 0;
  if (nMsgs >= MAX_OPS * 3)
    return 0;
  if ((temp = t->getAttributeContainer("use")) != 0)
    {
      use = *((string *) (temp->getValue()));
      if (use == "literal")
        m_body[nMsgs].use = LITERAL;

      else
        m_body[nMsgs].use = ENC;
    }

  else
    m_body[nMsgs].use = LITERAL;
  if ((temp = t->getAttributeContainer("namespace")) != 0)
    {
      string * s = (string *) (temp->getValue());
      buf = new char[s->length() + 1];
      strncpy(buf, s->c_str(), s->length() + 1);
      m_body[nMsgs].urn = buf;
    }

  else
    m_body[nMsgs].urn = 0;
  Qname body("body");
  idTable[idCounter].typeId = mySchemaParser->getElementType(body);
  idTable[idCounter].index = nMsgs;
  idCounter++;
  nMsgs++;
  return startId + idCounter - 1;
}


int  Soap::processFault(int parent, TypeContainer *)
{

  //TODO
  idCounter++;
  return startId + idCounter - 1;
}


int  Soap::processAddress(int parent, TypeContainer * t)
{
  TypeContainer * temp = 0;
  string location;
  if (nLocation >= MAX_PTYPES)
    return 0;
  if ((temp = t->getAttributeContainer("location")) != 0)
    {
      string * s = (string *) (temp->getValue());
      char *buf = new char[s->length() + 1];
      strncpy(buf, s->c_str(), s->length() + 1);
      m_location[nLocation] = buf;
    }
  Qname address("address");
  idTable[idCounter].typeId = mySchemaParser->getElementType(address);
  idTable[idCounter].index = nLocation;
  idCounter++;
  nLocation++;
  return startId + idCounter - 1;
}


int  Soap::processHeader(int parent, TypeContainer * t)
{
  TypeContainer * temp = 0;
  Qname msg;
  string ns, part;
  Qname header("header");
  int partType, partNs;                         //type and the schema id of the referenced part
  int msgId;
  if ((temp = t->getAttributeContainer("message")) != 0)
    {
      msg = *((Qname *) (temp->getValue()));
    }
  if ((temp = t->getAttributeContainer("namespace")) != 0)
    {
      ns = *((string *) (temp->getValue()));
    }
  if (!ns.empty())
    msg.setNamespace(ns);
  const Message *m = wParser->getMessage(msg);
  if (m == 0)
    {
      error("Unkown message " + msg.getLocalName());
      return 0;
    }
  if ((temp = t->getAttributeContainer("parts")) != 0)
    {
      part = *((string *) (temp->getValue()));  //this is actually NMTOKENS
    }
  partType = m->getPartType(part);
  partNs = m->getPartContentSchemaId(part);
  if (m->getPartRefType(part) == Elem)
    {
      Element * e =
        (wParser->getSchemaParser(partNs))->getElement(partType);
      partType = e->getType();
    }
  if (partType == 0)
    error("Unkown part type id");
  idTable[idCounter].typeId = mySchemaParser->getElementType(header);
  idTable[idCounter].index = nHeader;
  m_header[nHeader].schema = partNs;
  m_header[nHeader].typeId = partType;
  nHeader++;
  idCounter++;
  return startId + idCounter - 1;
}


void
Soap::getSoapOperationInfo(int elemId, char **soapAction, int &style)
{
  if (elemId - startId >= idCounter)            //invalid elem Id
    return;
  int opId = idTable[elemId - startId].index;
  *soapAction = m_ops[opId].soapAction;
  style = m_ops[opId].style;
} 

void  
Soap::getSoapBodyInfo(int elemId, char **ns, int &use)
{
  if (elemId - startId >= idCounter)            //invalid elem Id
    return;
  int bodyId = idTable[elemId - startId].index;
  *ns = m_body[bodyId].urn;
  use = m_body[bodyId].use;
} 

void
Soap::getSoapHeaderInfo(int elemId, int &schemaId, int &typeId)
{
  if (elemId - startId >= idCounter)            //invalid elem Id
    return;
  int headerId = idTable[elemId - startId].index;
  schemaId = m_header[headerId].schema;
  typeId = m_header[headerId].typeId;
} 

void  
Soap::getServiceLocation(int elemId, char **location)
{
  if (elemId - startId >= idCounter)            //invalid elem Id
    return;
  int locId = idTable[elemId - startId].index;
  *location = m_location[locId];
} 

bool 
Soap::isSoapBody(int elemId)
{
  Qname  body("body");
  if (elemId - startId >= idCounter)            //invalid elem Id
    return false;
  if (idTable[elemId - startId].typeId ==
      mySchemaParser->getElementType(body))
    return true;

  else
    return false;
}


bool 
Soap::isSoapHeader(int elemId)
{
  Qname  header("header");
  if (elemId - startId >= idCounter)            //invalid elem Id
    return false;
  if (idTable[elemId - startId].typeId ==
      mySchemaParser->getElementType(header))
    return true;

  else
    return false;
}


void  Soap::error(string s)
{
  cout << "Soap Processing" << dbsp << s << endl;
}
