/* 
 * wsdlpull - A C++ parser  for Wsdl  (Web services description language)
 * Copyright (C) 2003 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */

#include <wsdlparser/WsdlParser.h>

#ifdef LOGGING


// just print all the stuff,for debugging
void  WsdlParser::print(ostream & out)
{
  list < const Binding *>*pLBind = getBindings();
  const Binding *bn = pLBind->front();
  int i = 0, soapBindingId, inId, outId, faultId;
  int numOps = bn->numOps();

  /*
    Print all the  operations and messages and then the types of
    the Schema Parser
  */
  for (i = 0; i < nSchemas; i++)
    m_schemaParser[i]->print(out);
  i = 0;
  while (i < numOps)
    {
      const int *bindings = 0;
      int n = 0;
      n = bn->getOpBinding(i, &bindings);
      soapBindingId = bindings[0];
      n = bn->getInputBinding(i, &bindings);
      inId = bindings[0];
      n = bn->getOutputBinding(i, &bindings);
      outId = bindings[0];
      n = bn->getFaultBinding(i, &bindings);
      faultId = bindings[0];
      const Operation *op = bn->getOperation(i);
      out << op->getName() << endl;
      const Message *inMessage = op->getMessage(Input);
      const Message *outMessage = op->getMessage(Output);
      int nParams = 0, j = 0, k = 0;
      int typeId = 0, elemId = -1, schemaId;
      const XSDType *pType = 0;
      nParams = inMessage->getNumParts();
      for (j = 0; j < nParams; j++)
        {
          if (inMessage->getPartRefType(j) == Elem)
            {
              elemId = inMessage->getPartType(j);
              schemaId = inMessage->getPartContentSchemaId(j);
              Element * e =
                m_schemaParser[schemaId]->getElement(elemId);
              typeId = e->getType();
              pType = m_schemaParser[schemaId]->getType(typeId);

              if (!pType->isSimple())
                {
                  ComplexType * cType = (ComplexType *) pType;
                  for (k = 0; k < cType->getNumChildren(); k++)
                    out << cType->getChildName(k) << ":" << cType->
                      getChildType(k) << dbsp;
                }
            }

          else
            out << inMessage->getPartName(j) << ":" << inMessage->
              getPartType(j) << dbsp;
        }
      out << endl;
      nParams = outMessage->getNumParts();
      for (j = 0; j < nParams; j++)
        {
          if (outMessage->getPartRefType(j) == Elem)
            {
              elemId = outMessage->getPartType(j);
              schemaId = outMessage->getPartContentSchemaId(j);
              Element * e =
                m_schemaParser[schemaId]->getElement(elemId);
              out << e->getName() << ":" << e->getType() << dbsp;
            }

          else
            out << outMessage->getPartName(j) << ":" << outMessage->
              getPartType(j) << dbsp;
        }
      out << blk;
      i++;
    }
}
#endif
